{
  "cells": [
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# Boundary conditions tutorial\n",
        "\n",
        "This tutorial aims to demonstrate how users can implement various\n",
        "boundary conditions in Devito, building on concepts introduced in\n",
        "previous tutorials. Over the course of this notebook we will go over the\n",
        "implementation of both free surface boundary conditions and\n",
        "perfectly-matched layers (PMLs) in the context of the first-order\n",
        "acoustic wave equation. This tutorial is based on a simplified version\n",
        "of the method outlined in Liu and Tao’s 1997 paper\n",
        "(https://doi.org/10.1121/1.419657).\n",
        "\n",
        "We will set up our domain with PMLs along the left, right, and bottom\n",
        "edges, and free surface boundaries at the top as shown below.\n",
        "\n",
        "<img src=\"figures/boundary_conditions.png\" style=\"width: 220px;\"/>\n",
        "\n",
        "Note that whilst in practice we would want the PML tapers to overlap in\n",
        "the corners, this requires additional subdomains. As such, they are\n",
        "omitted for simplicity.\n",
        "\n",
        "As always, we will begin by specifying some parameters for our `Grid`:"
      ],
      "id": "90b15468-cca6-412c-a8e2-f54d4c2ce724"
    },
    {
      "cell_type": "code",
      "execution_count": 1,
      "metadata": {},
      "outputs": [],
      "source": [
        "%matplotlib inline\n",
        "\n",
        "shape = (101, 101)\n",
        "extent = (2000., 2000.)\n",
        "nbpml = 10  # Number of PMLs on each side"
      ],
      "id": "cell-3"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "We will need to use subdomains to accomodate the modified equations in\n",
        "the PML regions. As `Grid` objects cannot have subdomains added\n",
        "retroactively, we must define our subdomains beforehand."
      ],
      "id": "ecc2cf29-3e2e-40c5-8148-95e3bbacd555"
    },
    {
      "cell_type": "code",
      "execution_count": 2,
      "metadata": {},
      "outputs": [],
      "source": [
        "from devito import SubDomain\n",
        "\n",
        "s_o = 6  # Space order\n",
        "\n",
        "class MainDomain(SubDomain):  # Main section with no damping\n",
        "    name = 'main'\n",
        "    def __init__(self, PMLS, S_O):\n",
        "        super().__init__()\n",
        "        self.PMLS = PMLS\n",
        "        self.S_O = S_O\n",
        "\n",
        "    def define(self, dimensions):\n",
        "        x, y = dimensions\n",
        "        return {x: ('middle', self.PMLS, self.PMLS),\n",
        "                y: ('middle', self.S_O//2, self.PMLS)}\n",
        "\n",
        "\n",
        "class Left(SubDomain):  # Left PML region\n",
        "    name = 'left'\n",
        "    def __init__(self, PMLS):\n",
        "        super().__init__()\n",
        "        self.PMLS = PMLS\n",
        "\n",
        "    def define(self, dimensions):\n",
        "        x, y = dimensions\n",
        "        return {x: ('left', self.PMLS), y: y}\n",
        "\n",
        "\n",
        "class Right(SubDomain):  # Right PML region\n",
        "    name = 'right'\n",
        "    def __init__(self, PMLS):\n",
        "        super().__init__()\n",
        "        self.PMLS = PMLS\n",
        "\n",
        "    def define(self, dimensions):\n",
        "        x, y = dimensions\n",
        "        return {x: ('right', self.PMLS), y: y}\n",
        "    \n",
        "class Base(SubDomain):  # Base PML region\n",
        "    name = 'base'\n",
        "    def __init__(self, PMLS):\n",
        "        super().__init__()\n",
        "        self.PMLS = PMLS\n",
        "\n",
        "    def define(self, dimensions):\n",
        "        x, y = dimensions\n",
        "        return {x: ('middle', self.PMLS, self.PMLS), y: ('right', self.PMLS)}\n",
        "\n",
        "class FreeSurface(SubDomain):  # Free surface region\n",
        "    name = 'freesurface'\n",
        "    def __init__(self, PMLS, S_O):\n",
        "        super().__init__()\n",
        "        self.PMLS = PMLS\n",
        "        self.S_O = S_O\n",
        "        \n",
        "    def define(self, dimensions):\n",
        "        x, y = dimensions\n",
        "        return {x: ('middle', self.PMLS, self.PMLS), y: ('left', self.S_O//2)}\n",
        "\n",
        "main_domain = MainDomain(nbpml, s_o)\n",
        "left = Left(nbpml)\n",
        "right = Right(nbpml)\n",
        "base = Base(nbpml)\n",
        "freesurface = FreeSurface(nbpml, s_o)"
      ],
      "id": "cell-5"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "And create the grid, attaching our subdomains:"
      ],
      "id": "35bdd59f-0871-41d5-898a-2cf2b6abf7c9"
    },
    {
      "cell_type": "code",
      "execution_count": 3,
      "metadata": {},
      "outputs": [],
      "source": [
        "from devito import Grid\n",
        "\n",
        "grid = Grid(shape=shape, extent=extent,\n",
        "            subdomains=(main_domain, left, right, base, freesurface))\n",
        "x, y = grid.dimensions"
      ],
      "id": "cell-7"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "We can then begin to specify our problem starting with some parameters."
      ],
      "id": "e0eba214-c3bc-41b1-9936-244990371d68"
    },
    {
      "cell_type": "code",
      "execution_count": 4,
      "metadata": {},
      "outputs": [],
      "source": [
        "density = 1.  # 1000kg/m^3\n",
        "velocity = 4.  # km/s\n",
        "gamma = 0.0002  # Absorption coefficient"
      ],
      "id": "cell-9"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "We also need a `TimeFunction` object for each of our wavefields. As\n",
        "particle velocity is a vector, we will choose a `VectorTimeFunction`\n",
        "object to encapsulate it."
      ],
      "id": "f34cf894-1273-41be-a900-ad3cbfa0dbc3"
    },
    {
      "cell_type": "code",
      "execution_count": 5,
      "metadata": {},
      "outputs": [],
      "source": [
        "from devito import TimeFunction, VectorTimeFunction, NODE\n",
        "\n",
        "p = TimeFunction(name='p', grid=grid, time_order=1,\n",
        "                 space_order=s_o, staggered=NODE)\n",
        "v = VectorTimeFunction(name='v', grid=grid, time_order=1,\n",
        "                       space_order=s_o)"
      ],
      "id": "cell-11"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "A `VectorTimeFunction` is near identical in function to a standard\n",
        "`TimeFunction`, albeit with a field for each grid dimension. The fields\n",
        "associated with each component can be accessed as follows:"
      ],
      "id": "4180899a-f949-49b6-908f-20098e84bc98"
    },
    {
      "cell_type": "code",
      "execution_count": 6,
      "metadata": {},
      "outputs": [
        {
          "output_type": "stream",
          "name": "stdout",
          "text": [
            "[[[0. 0. 0. ... 0. 0. 0.]\n",
            "  [0. 0. 0. ... 0. 0. 0.]\n",
            "  [0. 0. 0. ... 0. 0. 0.]\n",
            "  ...\n",
            "  [0. 0. 0. ... 0. 0. 0.]\n",
            "  [0. 0. 0. ... 0. 0. 0.]\n",
            "  [0. 0. 0. ... 0. 0. 0.]]\n",
            "\n",
            " [[0. 0. 0. ... 0. 0. 0.]\n",
            "  [0. 0. 0. ... 0. 0. 0.]\n",
            "  [0. 0. 0. ... 0. 0. 0.]\n",
            "  ...\n",
            "  [0. 0. 0. ... 0. 0. 0.]\n",
            "  [0. 0. 0. ... 0. 0. 0.]\n",
            "  [0. 0. 0. ... 0. 0. 0.]]]"
          ]
        }
      ],
      "source": [
        "print(v[0].data)  # Print the data attribute associated with the x component of v"
      ],
      "id": "cell-13"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "You may have also noticed the keyword `staggered` in the arguments when\n",
        "we created these functions. As one might expect, these are used for\n",
        "specifying where derivatives should be evaluated relative to the grid,\n",
        "as required for implementing formulations such as the first-order\n",
        "acoustic wave equation or P-SV elastic. Passing a function\n",
        "`staggered=NODE` specifies that its derivatives should be evaluated at\n",
        "the node. One can also pass `staggered=x` or `staggered=y` to stagger\n",
        "the grid by half a spacing in those respective directions. Additionally,\n",
        "a tuple of dimensions can be passed to stagger in multiple directions\n",
        "(e.g. `staggered=(x, y)`). `VectorTimeFunction` objects have their\n",
        "associated grids staggered by default.\n",
        "\n",
        "We will also need to define a field for integrating pressure over time:"
      ],
      "id": "8487b608-e55b-4a94-9fcf-b77983f44b6d"
    },
    {
      "cell_type": "code",
      "execution_count": 7,
      "metadata": {},
      "outputs": [],
      "source": [
        "p_i = TimeFunction(name='p_i', grid=grid, time_order=1,\n",
        "                   space_order=1, staggered=NODE)"
      ],
      "id": "cell-15"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "Next we prepare the source term:"
      ],
      "id": "59913b3d-bf20-4534-861e-497bb73b609e"
    },
    {
      "cell_type": "code",
      "execution_count": 8,
      "metadata": {},
      "outputs": [],
      "source": [
        "import numpy as np\n",
        "from examples.seismic import TimeAxis, RickerSource\n",
        "\n",
        "t0 = 0.  # Simulation starts at t=0\n",
        "tn = 400.  # Simulation length in ms\n",
        "dt = 1e2*(1. / np.sqrt(2.)) / 60.  # Time step\n",
        "\n",
        "time_range = TimeAxis(start=t0, stop=tn, step=dt)\n",
        "\n",
        "f0 = 0.02\n",
        "src = RickerSource(name='src', grid=grid, f0=f0,\n",
        "                   npoint=1, time_range=time_range)\n",
        "\n",
        "# Position source centrally in all dimensions\n",
        "src.coordinates.data[0, :] = 1000."
      ],
      "id": "cell-17"
    },
    {
      "cell_type": "code",
      "execution_count": 9,
      "metadata": {
        "tags": [
          "nbval-skip"
        ]
      },
      "outputs": [
        {
          "output_type": "display_data",
          "metadata": {},
          "data": {
            "image/png": "iVBORw0KGgoAAAANSUhEUgAAAg0AAAF9CAYAAACH0lvIAAAAOXRFWHRTb2Z0d2FyZQBNYXRwbG90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\n"
          }
        }
      ],
      "source": [
        "src.show()"
      ],
      "id": "cell-18"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "For our PMLs, we will need some damping parameter. In this case, we will\n",
        "use a quadratic taper over the absorbing regions on the left and right\n",
        "sides of the domain."
      ],
      "id": "7e2a36e2-42bd-4f9b-9a7a-74cb5007a326"
    },
    {
      "cell_type": "code",
      "execution_count": 10,
      "metadata": {},
      "outputs": [],
      "source": [
        "# Damping parameterisation\n",
        "d_l = (1-0.1*x)**2  # Left side\n",
        "d_r = (1-0.1*(grid.shape[0]-1-x))**2  # Right side\n",
        "d_b = (1-0.1*(grid.shape[1]-1-y))**2  # Base edge"
      ],
      "id": "cell-20"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "Now for our main domain equations:"
      ],
      "id": "d3e7971e-5774-4aca-b7f8-447da37b3e83"
    },
    {
      "cell_type": "code",
      "execution_count": 11,
      "metadata": {},
      "outputs": [],
      "source": [
        "from devito import Eq, grad, div\n",
        "\n",
        "eq_v = Eq(v.forward, v - dt*grad(p)/density,\n",
        "          subdomain=grid.subdomains['main'])\n",
        "\n",
        "eq_p = Eq(p.forward, p - dt*velocity**2*density*div(v.forward),\n",
        "           subdomain=grid.subdomains['main'])"
      ],
      "id": "cell-22"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "We will also need to set up `p_i` to calculate the integral of `p` over\n",
        "time for out PMLs:"
      ],
      "id": "68fdcb9b-8539-4162-8b65-1ae5b6ba4217"
    },
    {
      "cell_type": "code",
      "execution_count": 12,
      "metadata": {},
      "outputs": [],
      "source": [
        "eq_p_i = Eq(p_i.forward, p_i + dt*(p.forward+p)/2)"
      ],
      "id": "cell-24"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "And add the equations for our damped region:"
      ],
      "id": "4729381a-3d08-41b1-863b-91d23b9b7aaf"
    },
    {
      "cell_type": "code",
      "execution_count": 13,
      "metadata": {},
      "outputs": [],
      "source": [
        "# Left side\n",
        "eq_v_damp_left = Eq(v.forward,\n",
        "                    (1-d_l)*v - dt*grad(p)/density,\n",
        "                    subdomain=grid.subdomains['left'])\n",
        "\n",
        "eq_p_damp_left = Eq(p.forward,\n",
        "                    (1-gamma*velocity**2*dt-d_l*dt)*p\n",
        "                    - d_l*gamma*velocity**2*p_i\n",
        "                    - dt*velocity**2*density*div(v.forward),\n",
        "                    subdomain=grid.subdomains['left'])\n",
        "\n",
        "# Right side\n",
        "eq_v_damp_right = Eq(v.forward,\n",
        "                     (1-d_r)*v - dt*grad(p)/density,\n",
        "                     subdomain=grid.subdomains['right'])\n",
        "\n",
        "eq_p_damp_right = Eq(p.forward,\n",
        "                     (1-gamma*velocity**2*dt-d_r*dt)*p\n",
        "                     - d_r*gamma*velocity**2*p_i\n",
        "                     - dt*velocity**2*density*div(v.forward),\n",
        "                     subdomain=grid.subdomains['right'])\n",
        "\n",
        "# Base edge\n",
        "eq_v_damp_base = Eq(v.forward,\n",
        "                    (1-d_b)*v - dt*grad(p)/density,\n",
        "                    subdomain=grid.subdomains['base'])\n",
        "\n",
        "eq_p_damp_base = Eq(p.forward,\n",
        "                    (1-gamma*velocity**2*dt-d_b*dt)*p\n",
        "                    - d_b*gamma*velocity**2*p_i\n",
        "                    - dt*velocity**2*density*div(v.forward),\n",
        "                    subdomain=grid.subdomains['base'])"
      ],
      "id": "cell-26"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "We next add our free surface boundary conditions. Note that this\n",
        "implementation is based off that found in\n",
        "`examples/seismic/acoustic/operators.py`."
      ],
      "id": "89005159-7e24-4d4f-b673-e2182f0f9193"
    },
    {
      "cell_type": "code",
      "execution_count": 14,
      "metadata": {},
      "outputs": [],
      "source": [
        "from devito import sign, norm\n",
        "from devito.symbolics import retrieve_functions, INT\n",
        "\n",
        "def free_surface_top(eq, subdomain, update):\n",
        "    \"\"\"\n",
        "    Modify a stencil such that it is folded back on\n",
        "    itself rather than leaving the model domain. This is\n",
        "    achieved by replacing the symbolic indices for some\n",
        "    function of those indices. Depending on how this is\n",
        "    done, this can be used to implement a pressure or\n",
        "    velocity free-surface. This is the MPI-safe method\n",
        "    of implementing a free-surface boundary condition\n",
        "    in Devito.\n",
        "    \n",
        "    Parameters\n",
        "    ----------\n",
        "    eq : Eq\n",
        "        The update stencil to modify\n",
        "    subdomain : FreeSurface\n",
        "        The subdomain in which the modification is\n",
        "        applied\n",
        "    update : str\n",
        "        The field being updated: 'pressure' or 'velocity'\n",
        "    \"\"\"\n",
        "    lhs, rhs = eq.evaluate.args\n",
        "    \n",
        "    # Get vertical subdimension and its parent\n",
        "    yfs = subdomain.dimensions[-1]\n",
        "    y = yfs.parent\n",
        "    \n",
        "    # Functions present in stencil\n",
        "    funcs = retrieve_functions(rhs)\n",
        "    mapper = {}\n",
        "    for f in funcs:\n",
        "        # Get the y index\n",
        "        yind = f.indices[-1]\n",
        "        if (yind - y).as_coeff_Mul()[0] < 0:\n",
        "            # If point position in stencil is negative\n",
        "            # Substitute the dimension for its subdomain companion\n",
        "            # Get the symbolic sign of this\n",
        "            s = sign(yind.subs({y: yfs, y.spacing: 1}))\n",
        "            if update == 'velocity':\n",
        "                # Antisymmetric mirror\n",
        "                # Substitute where index is negative for -ve where index is positive\n",
        "                mapper.update({f: s*f.subs({yind: INT(abs(yind))})})\n",
        "            elif update == 'pressure':\n",
        "                # Symmetric mirror\n",
        "                # Substitute where index is negative for +ve where index is positive\n",
        "                mapper.update({f: f.subs({yind: INT(abs(yind))})})\n",
        "    return Eq(lhs, rhs.subs(mapper), subdomain=subdomain)\n",
        "    \n",
        "fs_p = free_surface_top(eq_p, grid.subdomains['freesurface'], 'pressure')\n",
        "fs_v = free_surface_top(eq_v, grid.subdomains['freesurface'], 'velocity')"
      ],
      "id": "cell-28"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "And our source terms:"
      ],
      "id": "e07ad7da-2c86-42c8-9233-0480d33e8a51"
    },
    {
      "cell_type": "code",
      "execution_count": 15,
      "metadata": {},
      "outputs": [],
      "source": [
        "src_term = src.inject(field=p.forward, expr=src)"
      ],
      "id": "cell-30"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "Construct our operator and run:"
      ],
      "id": "f5702409-61b5-4832-9820-4f15cfe4e18a"
    },
    {
      "cell_type": "code",
      "execution_count": 16,
      "metadata": {
        "tags": [
          "nbval-ignore-output"
        ]
      },
      "outputs": [
        {
          "output_type": "stream",
          "name": "stderr",
          "text": [
            "Operator `Kernel` ran in 0.05 s"
          ]
        },
        {
          "output_type": "display_data",
          "metadata": {},
          "data": {
            "text/plain": [
              "PerformanceSummary([(PerfKey(name='section0', rank=None),\n",
              "                     PerfEntry(time=4.9e-05, gflopss=0.0, gpointss=0.0, oi=0.0, ops=0, itershapes=[])),\n",
              "                    (PerfKey(name='section1', rank=None),\n",
              "                     PerfEntry(time=0.0036749999999999908, gflopss=0.0, gpointss=0.0, oi=0.0, ops=0, itershapes=[])),\n",
              "                    (PerfKey(name='section2', rank=None),\n",
              "                     PerfEntry(time=0.013221000000000004, gflopss=0.0, gpointss=0.0, oi=0.0, ops=0, itershapes=[])),\n",
              "                    (PerfKey(name='section3', rank=None),\n",
              "                     PerfEntry(time=0.0037539999999999913, gflopss=0.0, gpointss=0.0, oi=0.0, ops=0, itershapes=[])),\n",
              "                    (PerfKey(name='section4', rank=None),\n",
              "                     PerfEntry(time=0.013060000000000006, gflopss=0.0, gpointss=0.0, oi=0.0, ops=0, itershapes=[])),\n",
              "                    (PerfKey(name='section5', rank=None),\n",
              "                     PerfEntry(time=0.003929000000000017, gflopss=0.0, gpointss=0.0, oi=0.0, ops=0, itershapes=[])),\n",
              "                    (PerfKey(name='section6', rank=None),\n",
              "                     PerfEntry(time=0.003147999999999996, gflopss=0.0, gpointss=0.0, oi=0.0, ops=0, itershapes=[]))])"
            ]
          }
        }
      ],
      "source": [
        "from devito import Operator\n",
        "\n",
        "op = Operator([eq_v, fs_v, eq_v_damp_left, eq_v_damp_base, eq_v_damp_right,\n",
        "               eq_p, fs_p, eq_p_damp_left, eq_p_damp_base, eq_p_damp_right,\n",
        "               eq_p_i]\n",
        "               + src_term)\n",
        "\n",
        "op(time=time_range.num-1)"
      ],
      "id": "cell-32"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "It is important to remember that the ordering of equations when an\n",
        "`Operator` is created dictates the order of loops within the generated c\n",
        "code. As such, the `v` equations all need to be placed before the `p`\n",
        "ones otherwise the operator will attempt to use the updated `v` fields\n",
        "before they have been updated.\n",
        "\n",
        "Now let’s plot the wavefield."
      ],
      "id": "21af46ce-888d-4bf2-a7bf-87b96587345c"
    },
    {
      "cell_type": "code",
      "execution_count": 17,
      "metadata": {
        "tags": [
          "nbval-skip"
        ]
      },
      "outputs": [
        {
          "output_type": "display_data",
          "metadata": {},
          "data": {
            "image/png": "iVBORw0KGgoAAAANSUhEUgAAAfUAAAGDCAYAAAAyM4nNAAAAOXRFWHRTb2Z0d2FyZQBNYXRwbG90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\nEGKCIKEuhBBCTBAk1IUQQogJwv8Hxxj+oeSPEB4AAAAASUVORK5CYII=\n"
          }
        }
      ],
      "source": [
        "import matplotlib.pyplot as plt\n",
        "\n",
        "scale = np.max(p.data[1])\n",
        "plt.imshow(p.data[1].T/scale,\n",
        "           origin=\"upper\",\n",
        "           vmin=-1, vmax=1,\n",
        "           extent=[0, grid.extent[0], grid.extent[1], 0],\n",
        "           cmap='seismic')\n",
        "plt.colorbar()\n",
        "plt.xlabel(\"x (m)\")\n",
        "plt.ylabel(\"y (m)\")\n",
        "plt.show()"
      ],
      "id": "cell-34"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "As we can see, the wave is effectively damped at the edge of the domain\n",
        "by the 10 layers of PMLs, with diminished reflections back into the\n",
        "domain."
      ],
      "id": "c3ce231a-1096-4066-b0f3-be40a32fb759"
    },
    {
      "cell_type": "code",
      "execution_count": 18,
      "metadata": {},
      "outputs": [],
      "source": [
        "assert(np.isclose(norm(v[0]), 0.1955, atol=0, rtol=1e-4))\n",
        "assert(np.isclose(norm(v[1]), 0.4596, atol=0, rtol=1e-4))\n",
        "assert(np.isclose(norm(p), 2.0043, atol=0, rtol=1e-4))"
      ],
      "id": "cell-36"
    }
  ],
  "nbformat": 4,
  "nbformat_minor": 5,
  "metadata": {
    "celltoolbar": "Tags",
    "kernelspec": {
      "name": "python3",
      "display_name": "Python 3 (ipykernel)",
      "language": "python"
    },
    "language_info": {
      "name": "python",
      "codemirror_mode": {
        "name": "ipython",
        "version": "3"
      },
      "file_extension": ".py",
      "mimetype": "text/x-python",
      "nbconvert_exporter": "python",
      "pygments_lexer": "ipython3",
      "version": "3.9.16"
    }
  }
}